/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QMap>
#include <math.h>
#include <locale.h>

#include "logger.h"
#include "flam3util.h"

namespace Util
{

	static QMap<QString, int> variations;


	const QMap<QString, int>& flam3_variations()
	{
		if (variations.isEmpty())
			for (int n = 0 ; n < flam3_nvariations ; n++)
				variations.insert(flam3_variation_names[n], n);
		return variations;
	}

	int variation_number(const char* var)
	{
		QString s(var);
		return variation_number( s );
	}

	int variation_number(const QString& s)
	{
		// reverse map variations,
		if (variations.isEmpty())
			flam3_variations();
		if (!variations.contains(s))
			return -1;
		return variations.value( s );
	}

	QColor get_xform_color(flam3_genome* g, flam3_xform* xform)
	{
		int idx = (int)(xform->color[0] * 255.0);
		double* p = g->palette[idx];
		return QColor::fromRgbF(p[0], p[1], p[2]);
	}

	QTextStream& operator<<( QTextStream& os, flam3_genome& g )
	{
		os  << "symmetry: "  << g.symmetry
			<< "\ntime: " << g.time
			<< "\nwidth, height: " << g.width << ", " << g.height
			<< "\ngenome has " << g.num_xforms << " xforms"
			<< "\nsample_density: " << g.sample_density
			<< "\nspatial_oversample: " << g.spatial_oversample
			<< "\nspatial_filter_radius: " << g.spatial_filter_radius
			<< "\ncenter: " << g.center[0] << ", " << g.center[1]
			<< "\nzoom: " << g.zoom;
		for (int n = 0 ; n < g.num_xforms ; n++)
			os << "\nxform " << n << " density: " << g.xform[n].density;
		os << endl;
		return os;
	}

	QTextStream& operator<<( QTextStream& os, flam3_frame& f )
	{
		os << "ngenomes: " << f.ngenomes
			<< "\nbits: " << f.bits
			<< "\npixel_aspect_ratio: " << f.pixel_aspect_ratio
			<< "\nverbose: " << f.verbose << "\n--genome--\n" << f.genomes;

		return os;
	}


	double get_xform_variable ( flam3_xform* xform, QString name )
	{
		if ( name == "blob_low" ) { return xform->blob_low ; }
		else if ( name == "blob_high" ) { return xform->blob_high ; }
		else if ( name == "blob_waves" ) { return xform->blob_waves ; }


		else if ( name == "pdj_a" ) { return xform->pdj_a ; }
		else if ( name == "pdj_b" ) { return xform->pdj_b ; }
		else if ( name == "pdj_c" ) { return xform->pdj_c ; }
		else if ( name == "pdj_d" ) { return xform->pdj_d ; }


		else if ( name == "fan2_x" ) { return xform->fan2_x ; }
		else if ( name == "fan2_y" ) { return xform->fan2_y ; }


		else if ( name == "rings2_val" ) { return xform->rings2_val ; }


		else if ( name == "perspective_angle" ) { return xform->perspective_angle ; }
		else if ( name == "perspective_dist" ) { return xform->perspective_dist ; }


		else if ( name == "juliaN_power" ) { return xform->juliaN_power ; }
		else if ( name == "juliaN_dist" ) { return xform->juliaN_dist ; }


		else if ( name == "juliaScope_power" ) { return xform->juliaScope_power ; }
		else if ( name == "juliaScope_dist" ) { return xform->juliaScope_dist ; }


		else if ( name == "radialBlur_angle" ) { return xform->radialBlur_angle ; }


		else if ( name == "pie_slices" ) { return xform->pie_slices ; }
		else if ( name == "pie_rotation" ) { return xform->pie_rotation ; }
		else if ( name == "pie_thickness" ) { return xform->pie_thickness ; }


		else if ( name == "ngon_sides" ) { return xform->ngon_sides ; }
		else if ( name == "ngon_power" ) { return xform->ngon_power ; }
		else if ( name == "ngon_circle" ) { return xform->ngon_circle ; }
		else if ( name == "ngon_corners" ) { return xform->ngon_corners ; }



		else if ( name == "curl_c1" ) { return xform->curl_c1 ; }
		else if ( name == "curl_c2" ) { return xform->curl_c2 ; }


		else if ( name == "rectangles_x" ) { return xform->rectangles_x ; }
		else if ( name == "rectangles_y" ) { return xform->rectangles_y ; }


		else if ( name == "amw_amp" ) { return xform->amw_amp ; }


		else if ( name == "disc2_rot" ) { return xform->disc2_rot ; }
		else if ( name == "disc2_twist" ) { return xform->disc2_twist ; }


		else if ( name == "supershape_rnd" ) { return xform->supershape_rnd ; }
		else if ( name == "supershape_m" ) { return xform->supershape_m ; }
		else if ( name == "supershape_n1" ) { return xform->supershape_n1 ; }
		else if ( name == "supershape_n2" ) { return xform->supershape_n2 ; }
		else if ( name == "supershape_n3" ) { return xform->supershape_n3 ; }
		else if ( name == "supershape_holes" ) { return xform->supershape_holes ; }


		else if ( name == "flower_petals" ) { return xform->flower_petals ; }
		else if ( name == "flower_holes" ) { return xform->flower_holes ; }


		else if ( name == "conic_eccen" ) { return xform->conic_eccen ; }
		else if ( name == "conic_holes" ) { return xform->conic_holes ; }


		else if ( name == "parabola_height" ) { return xform->parabola_height ; }
		else if ( name == "parabola_width" ) { return xform->parabola_width ; }


		else if ( name == "persp_vsin" ) { return xform->persp_vsin ; }
		else if ( name == "persp_vfcos" ) { return xform->persp_vfcos ; }


		else if ( name == "juliaN_rN" ) { return xform->juliaN_rN ; }
		else if ( name == "juliaN_cn" ) { return xform->juliaN_cn ; }


		else if ( name == "juliaScope_rN" ) { return xform->juliaScope_rN ; }
		else if ( name == "juliaScope_cn" ) { return xform->juliaScope_cn ; }


		else if ( name == "radialBlur_spinvar" ) { return xform->radialBlur_spinvar ; }
		else if ( name == "radialBlur_zoomvar" ) { return xform->radialBlur_zoomvar ; }


		else if ( name == "waves_dx2" ) { return xform->waves_dx2 ; }
		else if ( name == "waves_dy2" ) { return xform->waves_dy2 ; }


		else if ( name == "disc2_sinadd" ) { return xform->disc2_sinadd ; }
		else if ( name == "disc2_cosadd" ) { return xform->disc2_cosadd ; }
		else if ( name == "disc2_timespi" ) { return xform->disc2_timespi ; }

		else if ( name == "supershape_pm_4" ) { return xform->supershape_pm_4 ; }
		else if ( name == "supershape_pneg1_n1" ) { return xform->supershape_pneg1_n1 ; }

		return 0.;
	}

	void set_xform_variable ( flam3_xform* xform, QString name, double value )
	{
		if ( name == "blob_low" ) { xform->blob_low = value ; }
		else if ( name == "blob_high" ) { xform->blob_high = value ; }
		else if ( name == "blob_waves" ) { xform->blob_waves = value ; }


		else if ( name == "pdj_a" ) { xform->pdj_a = value ; }
		else if ( name == "pdj_b" ) { xform->pdj_b = value ; }
		else if ( name == "pdj_c" ) { xform->pdj_c = value ; }
		else if ( name == "pdj_d" ) { xform->pdj_d = value ; }


		else if ( name == "fan2_x" ) { xform->fan2_x = value ; }
		else if ( name == "fan2_y" ) { xform->fan2_y = value ; }


		else if ( name == "rings2_val" ) { xform->rings2_val = value ; }


		else if ( name == "perspective_angle" ) { xform->perspective_angle = value ; }
		else if ( name == "perspective_dist" ) { xform->perspective_dist = value ; }


		else if ( name == "juliaN_power" ) { xform->juliaN_power = value ; }
		else if ( name == "juliaN_dist" ) { xform->juliaN_dist = value ; }


		else if ( name == "juliaScope_power" ) { xform->juliaScope_power = value ; }
		else if ( name == "juliaScope_dist" ) { xform->juliaScope_dist = value ; }


		else if ( name == "radialBlur_angle" ) { xform->radialBlur_angle = value ; }


		else if ( name == "pie_slices" ) { xform->pie_slices = value ; }
		else if ( name == "pie_rotation" ) { xform->pie_rotation = value ; }
		else if ( name == "pie_thickness" ) { xform->pie_thickness = value ; }


		else if ( name == "ngon_sides" ) { xform->ngon_sides = value ; }
		else if ( name == "ngon_power" ) { xform->ngon_power = value ; }
		else if ( name == "ngon_circle" ) { xform->ngon_circle = value ; }
		else if ( name == "ngon_corners" ) { xform->ngon_corners = value ; }



		else if ( name == "curl_c1" ) { xform->curl_c1 = value ; }
		else if ( name == "curl_c2" ) { xform->curl_c2 = value ; }


		else if ( name == "rectangles_x" ) { xform->rectangles_x = value ; }
		else if ( name == "rectangles_y" ) { xform->rectangles_y = value ; }


		else if ( name == "amw_amp" ) { xform->amw_amp = value ; }


		else if ( name == "disc2_rot" ) { xform->disc2_rot = value ; }
		else if ( name == "disc2_twist" ) { xform->disc2_twist = value ; }


		else if ( name == "supershape_rnd" ) { xform->supershape_rnd = value ; }
		else if ( name == "supershape_m" ) { xform->supershape_m = value ; }
		else if ( name == "supershape_n1" ) { xform->supershape_n1 = value ; }
		else if ( name == "supershape_n2" ) { xform->supershape_n2 = value ; }
		else if ( name == "supershape_n3" ) { xform->supershape_n3 = value ; }
		else if ( name == "supershape_holes" ) { xform->supershape_holes = value ; }


		else if ( name == "flower_petals" ) { xform->flower_petals = value ; }
		else if ( name == "flower_holes" ) { xform->flower_holes = value ; }


		else if ( name == "conic_eccen" ) { xform->conic_eccen = value ; }
		else if ( name == "conic_holes" ) { xform->conic_holes = value ; }


		else if ( name == "parabola_height" ) { xform->parabola_height = value ; }
		else if ( name == "parabola_width" ) { xform->parabola_width = value ; }


		else if ( name == "persp_vsin" ) { xform->persp_vsin = value ; }
		else if ( name == "persp_vfcos" ) { xform->persp_vfcos = value ; }


		else if ( name == "juliaN_rN" ) { xform->juliaN_rN = value ; }
		else if ( name == "juliaN_cn" ) { xform->juliaN_cn = value ; }


		else if ( name == "juliaScope_rN" ) { xform->juliaScope_rN = value ; }
		else if ( name == "juliaScope_cn" ) { xform->juliaScope_cn = value ; }


		else if ( name == "radialBlur_spinvar" ) { xform->radialBlur_spinvar = value ; }
		else if ( name == "radialBlur_zoomvar" ) { xform->radialBlur_zoomvar = value ; }


		else if ( name == "waves_dx2" ) { xform->waves_dx2 = value ; }
		else if ( name == "waves_dy2" ) { xform->waves_dy2 = value ; }


		else if ( name == "disc2_sinadd" ) { xform->disc2_sinadd = value ; }
		else if ( name == "disc2_cosadd" ) { xform->disc2_cosadd = value ; }
		else if ( name == "disc2_timespi" ) { xform->disc2_timespi = value ; }

		else if ( name == "supershape_pm_4" ) { xform->supershape_pm_4 = value ; }
		else if ( name == "supershape_pneg1_n1" ) { xform->supershape_pneg1_n1 = value ; }

	}

	QStringList& get_variable_names()
	{
		static QStringList var_names = (QStringList()
				<< "blob_low"
				<< "blob_high"
				<< "blob_waves"

				<< "pdj_a"
				<< "pdj_b"
				<< "pdj_c"
				<< "pdj_d"

				<< "fan2_x"
				<< "fan2_y"

				<< "rings2_val"

				<< "perspective_angle"
				<< "perspective_dist"

				<< "juliaN_power"
				<< "juliaN_dist"

				<< "juliaScope_power"
				<< "juliaScope_dist"

				<< "radialBlur_angle"

				<< "pie_slices"
				<< "pie_rotation"
				<< "pie_thickness"

				<< "ngon_sides"
				<< "ngon_power"
				<< "ngon_circle"
				<< "ngon_corners"



				<< "curl_c1"
				<< "curl_c2"

				<< "rectangles_x"
				<< "rectangles_y"

				<< "amw_amp"

				<< "disc2_rot"
				<< "disc2_twist"

				<< "supershape_rnd"
				<< "supershape_m"
				<< "supershape_n1"
				<< "supershape_n2"
				<< "supershape_n3"
				<< "supershape_holes"

				<< "flower_petals"
				<< "flower_holes"

				<< "conic_eccen"
				<< "conic_holes"

				<< "parabola_height"
				<< "parabola_width"

	);

		return var_names;
	}

	char* setup_C_locale()
	{
		// force use of "C" locale when reading/writing reals.
		// first save away the current settings.
		char* locale = NULL;
		char* lorig  = setlocale(LC_NUMERIC, NULL);
		if (lorig == NULL)
			logError("Util::setup_C_locale : couldn't get current locale");
		else
		{
			int slen = strlen(lorig) + 1;
			locale = (char*)malloc(slen);
			if (locale != NULL)
				memcpy(locale, lorig, slen);
		}
		if (setlocale(LC_NUMERIC, "C") == NULL)
			logError("Util::setup_C_locale : couldn't set C locale");

		return locale;
	}

	void replace_C_locale(char* locale)
	{
		if (locale != NULL)
		{
			if (setlocale(LC_NUMERIC, locale) == NULL)
				logError("Util::replace_Ct_locale : couldn't replace locale settings");
			free(locale);
		}
	}

	void write_to_file(FILE* fd, flam3_genome* genome, char* attrs, int edits)
	{
		char* locale = setup_C_locale();
		flam3_print(fd, genome, attrs, edits);
		replace_C_locale(locale);
	}

	flam3_genome* read_from_file(FILE* fd, char* fn, int default_flag, int* ncps)
	{
		char* locale = setup_C_locale();
		flam3_genome* g = flam3_parse_from_file(fd, fn, default_flag, ncps);
		replace_C_locale(locale);
		return g;
	}

	flam3_genome* read_xml_string(QString xml, int* ncps)
	{
		char* locale = setup_C_locale();
		flam3_genome* g
			= flam3_parse_xml2(xml.toAscii().data(),
				QString("stdin").toAscii().data(), 1, ncps);
		replace_C_locale(locale);
		return g;
	}

	void init_genome(flam3_genome* g)
	{
		// copied from clear_cp() in flam.c in the libflam3 source.
		// maybe ask scott to export some of these more useful functions.
		//g->palette_index = flam3_palette_random;
		g->palette_index = flam3_palette_random;
		flam3_get_palette(flam3_palette_random, g->palette, 0.0);
		g->num_xforms = 0;
		g->xform = 0;
		g->time = 0.0;
		g->center[0] = 0.0;
		g->center[1] = 0.0;
		g->rot_center[0] = 0.0;
		g->rot_center[1] = 0.0;
		g->gamma = 4.0;
		g->vibrancy = 1.0;
		g->contrast = 1.0;
		g->brightness = 4.0;
		g->symmetry = 1;
		g->hue_rotation = 0.0;
		g->rotate = 0.0;
		g->edits = NULL;
		g->pixels_per_unit = 50;
		g->final_xform_enable = 0;
		g->final_xform_index = -1;
		g->interpolation = flam3_interpolation_linear;
		g->palette_interpolation = flam3_palette_interpolation_hsv;
		g->genome_index = 0;
		memset(g->parent_fname,0,flam3_parent_fn_len);
		memset(g->flame_name, 0, flam3_name_len+1);
		g->background[0] = 0.0;
		g->background[1] = 0.0;
		g->background[2] = 0.0;
		g->width = 100;
		g->height = 100;
		g->spatial_oversample = 1;
		g->spatial_filter_radius = 0.5;
		g->zoom = 0.0;
		g->sample_density = 1;
		/* Density estimation stuff defaulting to ON */
		g->estimator = 9.0;
		g->estimator_minimum = 0.0;
		g->estimator_curve = 0.4;
		g->gam_lin_thresh = 0.01;
		g->nbatches = 1;
		g->ntemporal_samples = 60;
		g->spatial_filter_select = flam3_gaussian_kernel;
	}

	void add_default_xform(flam3_genome* g)
	{
		int last = g->num_xforms;
		flam3_add_xforms(g, 1, 0);
		flam3_xform* xform = g->xform + last;

		xform->c[0][0] = 1.0;
		xform->c[0][1] = 0.0;
		xform->c[1][0] = 0.0;
		xform->c[1][1] = 1.0;
		xform->c[2][0] = 0.0;
		xform->c[2][1] = 0.0;

		for (int j = 0 ; j < flam3_nvariations ; j++)
			xform->var[j] = 0.0;
		xform->var[VAR_LINEAR] = 1.0;
		xform->color[0] = 0.0;
		xform->density = 0.5;
	}

#define PI 3.14159265358979323846

	void rectToPolar(double x, double y, double* r, double* p)
	{
		*r = sqrt(pow(x, 2.) + pow(y, 2.));
		if (x >= 0. && y >= 0.) *p = atan( y / x );
		else if (x < 0. && y >= 0.) *p = PI - atan( y / ((-1.)*x) );
		else if (x < 0. && y < 0.) *p = PI + atan( y / x );
		else *p = 2.*PI - atan( ((-1.)*y) / x );
	}

	void rectToPolarDeg(double x, double y, double* r, double* d)
	{
		rectToPolar(x, y, r, d);
		*d = *d * 180. / PI;
	}

	void polarToRect(double r, double p, double* x, double* y)
	{
		*x = r*cos(p);
		*y = r*sin(p);
	}

	void polarDegToRect(double r, double p, double* x, double* y)
	{
		polarToRect( r, p * PI / 180., x, y);
	}

}
