/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "xform.h"
#include <math.h>

namespace Lua
{
const char XForm::className[] = "XForm";
Lunar<XForm>::RegType XForm::methods[] =
{
	{ "index", &XForm::index },
	{ "density", &XForm::density },
	{ "symmetry", &XForm::symmetry },
	{ "color", &XForm::color },
	{ "var", &XForm::var },
	{ "param", &XForm::param },

	// xform coordinates
	{ "o", &XForm::a },
	{ "x", &XForm::b },
	{ "y", &XForm::c },
	{ "a", &XForm::a },
	{ "b", &XForm::b },
	{ "c", &XForm::c },
	{ "xa", &XForm::xa },
	{ "xb", &XForm::xb },
	{ "xc", &XForm::xc },
	{ "xd", &XForm::xd },
	{ "xe", &XForm::xe },
	{ "xf", &XForm::xf },
	{ "pos", &XForm::pos },
	{ "translate", &XForm::translate },
	{ "rotate", &XForm::rotate },
	{ "scale", &XForm::scale },
	{ "shear", &XForm::shear },

	// post xform coordinates
	{ "op", &XForm::ap },
	{ "xp", &XForm::bp },
	{ "yp", &XForm::cp },
	{ "ap", &XForm::ap },
	{ "bp", &XForm::bp },
	{ "cp", &XForm::cp },
	{ "xap", &XForm::xap },
	{ "xbp", &XForm::xbp },
	{ "xcp", &XForm::xcp },
	{ "xdp", &XForm::xdp },
	{ "xep", &XForm::xep },
	{ "xfp", &XForm::xfp },
	{ "posp", &XForm::posp },
	{ "translatep", &XForm::translatep },
	{ "rotatep", &XForm::rotatep },
	{ "scalep", &XForm::scalep },
	{ "shearp", &XForm::shearp },

	// params
	{ "param", &XForm::param },
	{ "precalc_sqrt_flag", &XForm::precalc_sqrt_flag },
	{ "precalc_angles_flag", &XForm::precalc_angles_flag },
	{ "precalc_atan_xy_flag", &XForm::precalc_atan_xy_flag },
	{ "precalc_atan_yx_flag", &XForm::precalc_atan_yx_flag },
	{ "blob_low", &XForm::blob_low },
	{ "blob_high", &XForm::blob_high },
	{ "blob_waves", &XForm::blob_waves },
	{ "pdj_a", &XForm::pdj_a },
	{ "pdj_b", &XForm::pdj_b },
	{ "pdj_c", &XForm::pdj_c },
	{ "pdj_d", &XForm::pdj_d },
	{ "fan2_x", &XForm::fan2_x },
	{ "fan2_y", &XForm::fan2_y },
	{ "rings2_val", &XForm::rings2_val },
	{ "perspective_angle", &XForm::perspective_angle },
	{ "perspective_dist", &XForm::perspective_dist },
	{ "juliaN_power", &XForm::juliaN_power },
	{ "juliaN_dist", &XForm::juliaN_dist },
	{ "juliaScope_power", &XForm::juliaScope_power },
	{ "juliaScope_dist", &XForm::juliaScope_dist },
	{ "radialBlur_angle", &XForm::radialBlur_angle },
	{ "pie_slices", &XForm::pie_slices },
	{ "pie_rotation", &XForm::pie_rotation },
	{ "pie_thickness", &XForm::pie_thickness },
	{ "ngon_sides", &XForm::ngon_sides },
	{ "ngon_power", &XForm::ngon_power },
	{ "ngon_circle", &XForm::ngon_circle },
	{ "ngon_corners", &XForm::ngon_corners },
	{ "curl_c1", &XForm::curl_c1 },
	{ "curl_c2", &XForm::curl_c2 },
	{ "rectangles_x", &XForm::rectangles_x },
	{ "rectangles_y", &XForm::rectangles_y },
	{ "amw_amp", &XForm::amw_amp },
	{ "disc2_rot", &XForm::disc2_rot },
	{ "disc2_twist", &XForm::disc2_twist },
	{ "supershape_rnd", &XForm::supershape_rnd },
	{ "supershape_m", &XForm::supershape_m },
	{ "supershape_n1", &XForm::supershape_n1 },
	{ "supershape_n2", &XForm::supershape_n2 },
	{ "supershape_n3", &XForm::supershape_n3 },
	{ "supershape_holes", &XForm::supershape_holes },
	{ "flower_petals", &XForm::flower_petals },
	{ "flower_holes", &XForm::flower_holes },
	{ "conic_eccen", &XForm::conic_eccen },
	{ "conic_holes", &XForm::conic_holes },
	{ "parabola_height", &XForm::parabola_height },
	{ "parabola_width", &XForm::parabola_width },
	{ "persp_vsin", &XForm::persp_vsin },
	{ "persp_vfcos", &XForm::persp_vfcos },
	{ "juliaN_rN", &XForm::juliaN_rN },
	{ "juliaN_cn", &XForm::juliaN_cn },
	{ "juliaScope_rN", &XForm::juliaScope_rN },
	{ "juliaScope_cn", &XForm::juliaScope_cn },
	{ "radialBlur_spinvar", &XForm::radialBlur_spinvar },
	{ "radialBlur_zoomvar", &XForm::radialBlur_zoomvar },
	{ "waves_dx2", &XForm::waves_dx2 },
	{ "waves_dy2", &XForm::waves_dy2 },
	{ "disc2_sinadd", &XForm::disc2_sinadd },
	{ "disc2_cosadd", &XForm::disc2_cosadd },
	{ "disc2_timespi", &XForm::disc2_timespi },
	{ "supershape_pm_4", &XForm::supershape_pm_4 },
	{ "supershape_pneg1_n1", &XForm::supershape_pneg1_n1 },
	{ 0, 0 }
};

XForm::XForm(lua_State* /*L*/)
{
}

XForm::~XForm()
{
}

#define intAcc(name) int XForm::name(lua_State *L)\
{\
	if (lua_gettop(L) == 1)\
		xform_ptr->name = luaL_checkint(L, 1);\
	else\
	{\
		lua_settop(L, 0);\
		lua_pushinteger(L, xform_ptr->name);\
	}\
	return 1;\
}\

#define intAccRO(name) int XForm::name(lua_State *L)\
{\
	lua_settop(L, 0);\
	lua_pushinteger(L, xform_ptr->name);\
	return 1;\
}\

#define realAcc(name) int XForm::name(lua_State *L)\
{\
	if (lua_gettop(L) == 1)\
		xform_ptr->name = luaL_checknumber(L, 1);\
	else\
	{\
		lua_settop(L, 0);\
		lua_pushnumber(L, xform_ptr->name);\
	}\
	return 1;\
}\

#define realAccRO(name) int XForm::name(lua_State *L)\
{\
	lua_settop(L, 0);\
	lua_pushnumber(L, xform_ptr->name);\
	return 1;\
}\

int XForm::index(lua_State* L)
{
	lua_settop(L, 0);
	lua_pushinteger(L, idx);
	return 1;
}

realAcc(density);
realAcc(symmetry);

int XForm::color(lua_State *L)
{
	if (lua_gettop(L) == 1)
		xform_ptr->color[0] = luaL_checknumber(L, 1);
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, xform_ptr->color[0]);
	}
	return 1;
}

int XForm::var(lua_State *L)
{
	int var_num;
	if (lua_isnumber(L, 1) == 1)
		var_num = lua_tointeger(L, 1);
	else
	{
		const char* name = luaL_checkstring(L, 1);
		var_num = Util::variation_number(QString(name));
		if (var_num == -1)
			luaL_error(L, "variation %s not found", name);
	}

	if (lua_gettop(L) == 2)
	{
		xform_ptr->var[var_num] = luaL_checknumber(L, 2);
		lua_remove(L, 1);
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, xform_ptr->var[var_num]);
	}

	return 1;
}

int XForm::param(lua_State *L)
{
	return var(L);
}

int XForm::a(lua_State *L)
{
	if (lua_gettop(L) == 2)
	{
		coords[0].rx() = luaL_checknumber(L, 1);
		coords[0].ry() = luaL_checknumber(L, 2);
		c2xf();
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, coords[0].x());
		lua_pushnumber(L, coords[0].y());
	}
	return 2;
}

int XForm::b(lua_State *L)
{
	if (lua_gettop(L) == 2)
	{
		coords[1].rx() = luaL_checknumber(L, 1);
		coords[1].ry() = luaL_checknumber(L, 2);
		c2xf();
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, coords[1].x());
		lua_pushnumber(L, coords[1].y());
	}
	return 2;
}

int XForm::c(lua_State *L)
{
	if (lua_gettop(L) == 2)
	{
		coords[2].rx() = luaL_checknumber(L, 1);
		coords[2].ry() = luaL_checknumber(L, 2);
		c2xf();
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, coords[2].x());
		lua_pushnumber(L, coords[2].y());
	}
	return 2;
}

int XForm::xa(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->c[0][0] = luaL_checknumber(L, 1);
		xf2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->c[0][0]);
	return 1;
}

int XForm::xd(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->c[0][1] = luaL_checknumber(L, 1);
		xf2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->c[0][1]);
	return 1;
}

int XForm::xb(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->c[1][0] = luaL_checknumber(L, 1);
		xf2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->c[1][0]);
	return 1;
}

int XForm::xe(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->c[1][1] = luaL_checknumber(L, 1);
		xf2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->c[1][1]);
	return 1;
}

int XForm::xc(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->c[2][0] = luaL_checknumber(L, 1);
		xf2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->c[2][0]);
	return 1;
}

int XForm::xf(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->c[2][1] = luaL_checknumber(L, 1);
		xf2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->c[2][1]);
	return 1;
}


// post xform accessors
int XForm::ap(lua_State *L)
{
	if (lua_gettop(L) == 2)
	{
		coordsp[0].rx() = luaL_checknumber(L, 1);
		coordsp[0].ry() = luaL_checknumber(L, 2);
		c2xfp();
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, coordsp[0].x());
		lua_pushnumber(L, coordsp[0].y());
	}
	return 2;
}

int XForm::bp(lua_State *L)
{
	if (lua_gettop(L) == 2)
	{
		coordsp[1].rx() = luaL_checknumber(L, 1);
		coordsp[1].ry() = luaL_checknumber(L, 2);
		c2xfp();
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, coordsp[1].x());
		lua_pushnumber(L, coordsp[1].y());
	}
	return 2;
}

int XForm::cp(lua_State *L)
{
	if (lua_gettop(L) == 2)
	{
		coordsp[2].rx() = luaL_checknumber(L, 1);
		coordsp[2].ry() = luaL_checknumber(L, 2);
		c2xfp();
	}
	else
	{
		lua_settop(L, 0);
		lua_pushnumber(L, coordsp[2].x());
		lua_pushnumber(L, coordsp[2].y());
	}
	return 2;
}

int XForm::xap(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->post[0][0] = luaL_checknumber(L, 1);
		xfp2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->post[0][0]);
	return 1;
}

int XForm::xdp(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->post[0][1] = luaL_checknumber(L, 1);
		xfp2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->post[0][1]);
	return 1;
}

int XForm::xbp(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->post[1][0] = luaL_checknumber(L, 1);
		xfp2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->post[1][0]);
	return 1;
}

int XForm::xep(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->post[1][1] = luaL_checknumber(L, 1);
		xfp2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->post[1][1]);
	return 1;
}

int XForm::xcp(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->post[2][0] = luaL_checknumber(L, 1);
		xfp2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->post[2][0]);
	return 1;
}

int XForm::xfp(lua_State *L)
{
	if (lua_gettop(L) == 1)
	{
		xform_ptr->post[2][1] = luaL_checknumber(L, 1);
		xfp2c();
	}
	lua_settop(L, 0);
	lua_pushnumber(L, xform_ptr->post[2][1]);
	return 1;
}

// param accessors
intAcc(precalc_sqrt_flag);
intAcc(precalc_angles_flag);
intAcc(precalc_atan_xy_flag);
intAcc(precalc_atan_yx_flag);
realAcc(blob_low);
realAcc(blob_high);
realAcc(blob_waves);
realAcc(pdj_a);
realAcc(pdj_b);
realAcc(pdj_c);
realAcc(pdj_d);
realAcc(fan2_x);
realAcc(fan2_y);
realAcc(rings2_val);
realAcc(perspective_angle);
realAcc(perspective_dist);
realAcc(juliaN_power);
realAcc(juliaN_dist);
realAcc(juliaScope_power);
realAcc(juliaScope_dist);
realAcc(radialBlur_angle);
realAcc(pie_slices);
realAcc(pie_rotation);
realAcc(pie_thickness);
realAcc(ngon_sides);
realAcc(ngon_power);
realAcc(ngon_circle);
realAcc(ngon_corners);
realAcc(curl_c1);
realAcc(curl_c2);
realAcc(rectangles_x);
realAcc(rectangles_y);
realAcc(amw_amp);
realAcc(disc2_rot);
realAcc(disc2_twist);
realAcc(supershape_rnd);
realAcc(supershape_m);
realAcc(supershape_n1);
realAcc(supershape_n2);
realAcc(supershape_n3);
realAcc(supershape_holes);
realAcc(flower_petals);
realAcc(flower_holes);
realAcc(conic_eccen);
realAcc(conic_holes);
realAcc(parabola_height);
realAcc(parabola_width);
realAcc(persp_vsin);
realAcc(persp_vfcos);
realAcc(juliaN_rN);
realAcc(juliaN_cn);
realAcc(juliaScope_rN);
realAcc(juliaScope_cn);
realAcc(radialBlur_spinvar);
realAcc(radialBlur_zoomvar);
realAcc(waves_dx2);
realAcc(waves_dy2);
realAcc(disc2_sinadd);
realAcc(disc2_cosadd);
realAcc(disc2_timespi);
realAcc(supershape_pm_4);
realAcc(supershape_pneg1_n1);


// c++ interface
void XForm::xf2c()
{
	coords = basisTriangle->getCoords(xform_ptr->c);
}

void XForm::c2xf()
{
	basisTriangle->applyTransform(coords, xform_ptr->c);
}

void XForm::xfp2c()
{
	coordsp = basisTriangle->getCoords(xform_ptr->post);
}

void XForm::c2xfp()
{
	basisTriangle->applyTransform(coordsp, xform_ptr->post);
}

void XForm::setContext(LuaThreadAdapter* ctx, int g_idx, int x_idx)
{
	LuaType::setContext(ctx);

	idx = x_idx;
	basisTriangle = m_adapter->basisTriangle();
	xform_ptr = (m_adapter->genomeVector()->data() + g_idx)->xform + x_idx;
	xf2c(); // load Ax,Ay, Bx,By, Cx,Cy
	xfp2c();
}

flam3_xform* XForm::data()
{
	return xform_ptr;
}

int XForm::translate(lua_State* L)
{
	double dx = luaL_checknumber(L, 1);
	double dy = luaL_checknumber(L, 2);
	coords.translate(dx, dy);
	c2xf();
	return 0;
}

int XForm::pos(lua_State* L)
{
	if (lua_gettop(L) == 2)
	{
		double x = luaL_checknumber(L, 1);
		double y = luaL_checknumber(L, 2);
		QPointF f = QPointF(x,y) - coords.boundingRect().center();
		coords.translate(f.x(), f.y());
		c2xf();
	}
	QPointF c = coords.boundingRect().center();
	lua_settop(L,0);
	lua_pushnumber(L, c.x());
	lua_pushnumber(L, c.y());
	return 2;
}

int XForm::rotate(lua_State* L)
{
	double deg = luaL_checknumber(L, 1);
	QPointF c(coords.boundingRect().center());
	if (lua_gettop(L) > 1)
	{
		double x = luaL_checknumber(L, 2);
		double y = luaL_checknumber(L, 3);
		c = QPointF(x, y);
	}
	QMatrix matrix;
	matrix.translate(c.x(), c.y()).rotate(deg).translate(-c.x(), -c.y());
	coords = matrix.map(QPolygonF(coords));
	c2xf();
	return 0;
}

int XForm::scale(lua_State* L)
{
	double dx = luaL_checknumber(L, 1);
	double dy = dx;
	if (lua_gettop(L) > 1)
		dy = luaL_checknumber(L, 2);
	QPointF c = coords.boundingRect().center();
	if (lua_gettop(L) > 2)
	{
		double x = luaL_checknumber(L, 3);
		double y = luaL_checknumber(L, 4);
		c = QPointF(x, y);
	}
	QMatrix matrix;
	matrix.translate(c.x(), c.y()).scale(dx, dy).translate(-c.x(), -c.y());
	coords = matrix.map(QPolygonF(coords));
	c2xf();
	return 0;
}

int XForm::shear(lua_State* L)
{
	double dx = luaL_checknumber(L, 1);
	double dy = luaL_checknumber(L, 2);
	QPointF c = coords.boundingRect().center();
	if (lua_gettop(L) > 2)
	{
		double x = luaL_checknumber(L, 3);
		double y = luaL_checknumber(L, 4);
		c = QPointF(x, y);
	}
	QMatrix matrix;
	matrix.translate(c.x(), c.y()).shear(dx, dy).translate(-c.x(), -c.y());
	coords = matrix.map(QPolygonF(coords));
	c2xf();
	return 0;
}

// post transform 'transforms'
int XForm::translatep(lua_State* L)
{
	double dx = luaL_checknumber(L, 1);
	double dy = luaL_checknumber(L, 2);
	coordsp.translate(dx, dy);
	c2xfp();
	return 0;
}

int XForm::posp(lua_State* L)
{
	if (lua_gettop(L) == 2)
	{
		double x = luaL_checknumber(L, 1);
		double y = luaL_checknumber(L, 2);
		QPointF f = QPointF(x,y) - coordsp.boundingRect().center();
		coordsp.translate(f.x(), f.y());
		c2xfp();
	}
	QPointF c = coordsp.boundingRect().center();
	lua_settop(L,0);
	lua_pushnumber(L, c.x());
	lua_pushnumber(L, c.y());
	return 2;
}

int XForm::rotatep(lua_State* L)
{
	double deg = luaL_checknumber(L, 1);
	QPointF c(coordsp.boundingRect().center());
	if (lua_gettop(L) > 1)
	{
		double x = luaL_checknumber(L, 2);
		double y = luaL_checknumber(L, 3);
		c = QPointF(x, y);
	}
	QMatrix matrix;
	matrix.translate(c.x(), c.y()).rotate(deg).translate(-c.x(), -c.y());
	coordsp = matrix.map(QPolygonF(coordsp));
	c2xfp();
	return 0;
}

int XForm::scalep(lua_State* L)
{
	double dx = luaL_checknumber(L, 1);
	double dy = dx;
	if (lua_gettop(L) > 1)
		dy = luaL_checknumber(L, 2);
	QPointF c = coordsp.boundingRect().center();
	if (lua_gettop(L) > 2)
	{
		double x = luaL_checknumber(L, 3);
		double y = luaL_checknumber(L, 4);
		c = QPointF(x, y);
	}
	QMatrix matrix;
	matrix.translate(c.x(), c.y()).scale(dx, dy).translate(-c.x(), -c.y());
	coordsp = matrix.map(QPolygonF(coordsp));
	c2xfp();
	return 0;
}

int XForm::shearp(lua_State* L)
{
	double dx = luaL_checknumber(L, 1);
	double dy = luaL_checknumber(L, 2);
	QPointF c = coordsp.boundingRect().center();
	if (lua_gettop(L) > 2)
	{
		double x = luaL_checknumber(L, 3);
		double y = luaL_checknumber(L, 4);
		c = QPointF(x, y);
	}
	QMatrix matrix;
	matrix.translate(c.x(), c.y()).shear(dx, dy).translate(-c.x(), -c.y());
	coordsp = matrix.map(QPolygonF(coordsp));
	c2xfp();
	return 0;
}

}
