/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef XFORM_LUA_H
#define XFORM_LUA_H

#include "lunar.h"
#include "basistriangle.h"
#include "luatype.h"

#define intAcc_H(name) int name(lua_State *L);
#define intAcc_HRO(name) int name(lua_State *L);
#define realAcc_H(name) int name(lua_State *L);
#define realAcc_HRO(name) int name(lua_State *L);

namespace Lua
{
class XForm : public LuaType
{
	int idx;
	TriangleCoords coords;
	TriangleCoords coordsp;
	BasisTriangle* basisTriangle;
	flam3_xform* xform_ptr;

	void c2xf();
	void c2xfp();
	void xf2c();
	void xfp2c();


	public:
		XForm(lua_State *L);
		~XForm();
		int index(lua_State* L);
		int density(lua_State *L);
		int symmetry(lua_State *L);
		int color(lua_State *L);
		int var(lua_State *L);
		int param(lua_State *L);
		int a(lua_State *L);
		int b(lua_State *L);
		int c(lua_State *L);
		int pos(lua_State* L);
		int translate(lua_State* L);
		int rotate(lua_State* L);
		int scale(lua_State* L);
		int shear(lua_State* L);

		int xa(lua_State *L);
		int xb(lua_State *L);
		int xc(lua_State *L);
		int xd(lua_State *L);
		int xe(lua_State *L);
		int xf(lua_State *L);

		int ap(lua_State *L);
		int bp(lua_State *L);
		int cp(lua_State *L);
		int posp(lua_State* L);
		int translatep(lua_State* L);
		int rotatep(lua_State* L);
		int scalep(lua_State* L);
		int shearp(lua_State* L);

		int xap(lua_State *L);
		int xbp(lua_State *L);
		int xcp(lua_State *L);
		int xdp(lua_State *L);
		int xep(lua_State *L);
		int xfp(lua_State *L);

		intAcc_H(precalc_sqrt_flag);
		intAcc_H(precalc_angles_flag);
		intAcc_H(precalc_atan_xy_flag);
		intAcc_H(precalc_atan_yx_flag);
		realAcc_H(blob_low);
		realAcc_H(blob_high);
		realAcc_H(blob_waves);
		realAcc_H(pdj_a);
		realAcc_H(pdj_b);
		realAcc_H(pdj_c);
		realAcc_H(pdj_d);
		realAcc_H(fan2_x);
		realAcc_H(fan2_y);
		realAcc_H(rings2_val);
		realAcc_H(perspective_angle);
		realAcc_H(perspective_dist);
		realAcc_H(juliaN_power);
		realAcc_H(juliaN_dist);
		realAcc_H(juliaScope_power);
		realAcc_H(juliaScope_dist);
		realAcc_H(radialBlur_angle);
		realAcc_H(pie_slices);
		realAcc_H(pie_rotation);
		realAcc_H(pie_thickness);
		realAcc_H(ngon_sides);
		realAcc_H(ngon_power);
		realAcc_H(ngon_circle);
		realAcc_H(ngon_corners);
		realAcc_H(curl_c1);
		realAcc_H(curl_c2);
		realAcc_H(rectangles_x);
		realAcc_H(rectangles_y);
		realAcc_H(amw_amp);
		realAcc_H(disc2_rot);
		realAcc_H(disc2_twist);
		realAcc_H(supershape_rnd);
		realAcc_H(supershape_m);
		realAcc_H(supershape_n1);
		realAcc_H(supershape_n2);
		realAcc_H(supershape_n3);
		realAcc_H(supershape_holes);
		realAcc_H(flower_petals);
		realAcc_H(flower_holes);
		realAcc_H(conic_eccen);
		realAcc_H(conic_holes);
		realAcc_H(parabola_height);
		realAcc_H(parabola_width);
		realAcc_H(persp_vsin);
		realAcc_H(persp_vfcos);
		realAcc_H(juliaN_rN);
		realAcc_H(juliaN_cn);
		realAcc_H(juliaScope_rN);
		realAcc_H(juliaScope_cn);
		realAcc_H(radialBlur_spinvar);
		realAcc_H(radialBlur_zoomvar);
		realAcc_H(waves_dx2);
		realAcc_H(waves_dy2);
		realAcc_H(disc2_sinadd);
		realAcc_H(disc2_cosadd);
		realAcc_H(disc2_timespi);
		realAcc_H(supershape_pm_4);
		realAcc_H(supershape_pneg1_n1);

		void setContext(LuaThreadAdapter*, int, int);
		flam3_xform* data();

		static const char className[];
		static Lunar<XForm>::RegType methods[];
};
}
#endif
