/***************************************************************************
 *   Copyright (C) 2007, 2008, 2009 by David Bitseff                       *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "undoring.h"
#include "logger.h"

UndoRing::UndoRing()
{
	logFiner("UndoRing::UndoRing() : enter");
	current = head = -1;
}

UndoRing::UndoRing(const UndoRing& in)
{
	logFiner("UndoRing::UndoRing(&copy) : enter");
	UndoRing();
	*this = in;
}

UndoRing& UndoRing::operator=(const UndoRing& in)
{
	for (int n = 0 ; n < UNDORING_SIZE ; n++)
		ring[n] = in.ring[n];
	current = in.current;
	head = in.head;
	return *this;
}

UndoRing::~UndoRing()
{
	clear();
}

UndoState* UndoRing::next()
{
	if (current < head)
		current++;
	logFine(QString("UndoRing::next : current %1 / [%2,%3]")
			.arg(current)
			.arg(head - UNDORING_SIZE)
			.arg(head));
	return ring + (current % UNDORING_SIZE);
}

UndoState* UndoRing::prev()
{
	if (current > 0)
	{
		if (current > (head - UNDORING_SIZE + 1))
			current--;
		logFine(QString("UndoRing::prev : current %1 / [%2,%3]")
			.arg(current)
			.arg(head - UNDORING_SIZE)
			.arg(head));
		return ring + (current % UNDORING_SIZE);
	}
	return ring;
}

UndoState* UndoRing::advance()
{
	current++;
	head = current;
	logFine(QString("UndoRing::advance : current %1").arg(current));
	return ring + (current % UNDORING_SIZE);
}

UndoState* UndoRing::currentState()
{
	return ring + (current % UNDORING_SIZE);
}

void UndoRing::clear()
{
	logFiner(QString("UndoRing::clear : enter"));
	for (int n = 0 ; n < UNDORING_SIZE ; n++)
		(ring + n)->clear();
	current = head = -1;
}

bool UndoRing::atHead()
{
	return current == head;
}

bool UndoRing::atTail()
{
	return current == (head - UNDORING_SIZE + 1) || current <= 0;
}

int UndoRing::index()
{
	return current;
}

int UndoRing::size()
{
	return UNDORING_SIZE;
}


UndoState::UndoState()
{
	Genome.xform = 0;
	Genome.num_xforms = 0;
	clear();
}

UndoState::UndoState(const UndoState& in)
{
	UndoState();
	*this = in;
}

UndoState& UndoState::operator=(const UndoState& in)
{
	flam3_copy(&Genome, (flam3_genome*)&(in.Genome)); // danger?
	SelectionRect = in.SelectionRect;
	SelectedType  = in.SelectedType;
	NodesO = in.NodesO;
	NodesX = in.NodesX;
	NodesY = in.NodesY;
	Triangles = in.Triangles;
	MarkPos   = in.MarkPos;
	return *this;
}

void UndoState::clear()
{
	if (Genome.xform && (Genome.num_xforms > 0))
		while (Genome.num_xforms > 0)
			flam3_delete_xform(&Genome, Genome.num_xforms - 1);
	memset(&Genome, 0, sizeof(flam3_genome));
	SelectionRect = QPolygonF();
	MarkPos = QPointF();
	NodesO.clear();
	NodesX.clear();
	NodesY.clear();
}
